/* Nessus
 * Copyright (C) 1999 - 2001 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * In addition, as a special exception, Renaud Deraison
 * gives permission to link the code of this program with any
 * version of the OpenSSL library which is distributed under a
 * license identical to that listed in the included COPYING.OpenSSL
 * file, and distribute linked combinations including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * this file, you may extend this exception to your version of the
 * file, but you are not obligated to do so.  If you do not wish to
 * do so, delete this exception statement from your version.
 *
 * Old style nessus plugins, implemented as shared libraries.
 *
 */

#include <includes.h>
#include "pluginload.h"
#include "threads.h"
#include "log.h"
#include "preferences.h"

static int nes_thread(struct arglist *);

#ifdef HAVE_SHL_LOAD	/* this is HP/UX */
ext_library_t dlopen(name)
 char* name;
{
 return (ext_library_t)shl_load(name, BIND_IMMEDIATE|BIND_VERBOSE|BIND_NOSTART, 0L);
}

void * 
dlsym(lib, name)
 shl_t lib;
 char * name;
{
 void* ret;
 int status;
 status = shl_findsym((shl_t *)&lib, name, TYPE_PROCEDURE, &ret);
 if((status == -1) && (errno == 0))
 {
  status = shl_findsym((shl_t *)&lib, name, TYPE_DATA, &ret);
 }
 
 return (status == -1 ) ? NULL : ret;
}

void
dlclose(x)
 shl_t * x;
{
 shl_unload(x);
}

char*
dlerror()
{
 return strerror(errno);
}

#else /* HAVE_SHL_LOAD */
#ifdef HAVE_NSCREATEOBJECTFILEIMAGEFROMFILE /* Darwin */
#ifdef HAVE_DL_LIB /* dlcompat is installed - not good */
#define dlopen macosx_dlopen
#define dlsym macosx_dlsym
#define dlclose macosx_dlclose
#define dlerror macosx_dlerror
#undef HAVE_DL_LIB
#endif
#include <mach-o/dyld.h>

ext_library_t
dlopen(name)
 char* name;
{
 NSObjectFileImage ofile;
 NSSymbol nsSymbol;
 
 if(NSCreateObjectFileImageFromFile(name, &ofile) != NSObjectFileImageSuccess)
 {
  fprintf(stderr, "NSCreateObjectFileImageFromFile(%s) failed\n", name);
  return NULL;
 }

 return NSLinkModule(ofile, name, NSLINKMODULE_OPTION_PRIVATE|
				  NSLINKMODULE_OPTION_BINDNOW);
}

void * 
dlsym(lib, name)
 void* lib;
 char * name;
{
 NSSymbol nsSymbol = NSLookupSymbolInModule((NSModule)lib, name); 
 if(nsSymbol == NULL)
 {
  /* fprintf(stderr, "NSLookupSymbolInModule(%x, %s) failed\n", lib, name); */
  return NULL;
 }
 return NSAddressOfSymbol(nsSymbol);
}

void
dlclose(x)
 void * x;
{
 NSUnLinkModule((NSModule)(x), NSUNLINKMODULE_OPTION_NONE);
}

char*
dlerror()
{
 return strerror(errno);
}
#endif /* Darwin */
#endif

/*
 *  Initialize this class
 */
pl_class_t* nes_plugin_init(struct arglist* prefs, struct arglist* args) {
    return &nes_plugin_class;
}

/*
 * add *one* .nes (shared lib) plugin to the server list
 */
struct arglist * 
nes_plugin_add(folder, name, plugins, preferences)
     char * folder;
     char * name;
     struct arglist * plugins;
     struct arglist * preferences;
{
 ext_library_t ptr = NULL; 
 char * fullname = NULL;
 struct arglist * plugin = NULL;
 struct arglist * prev_plugin = NULL;
 

  fullname = emalloc(strlen(folder)+strlen(name)+2);
  sprintf(fullname, "%s/%s", folder, name);

  if((ptr = LOAD_LIBRARY(fullname))==NULL){
    log_write("Couldn't load %s - %s\n", name, LIB_LAST_ERROR());
  }
  else {
    plugin_init_t  f_init;
    struct arglist * args;
    if((f_init = (plugin_init_t)LOAD_FUNCTION(ptr, "plugin_init")) ||
    	(f_init = (plugin_init_t)LOAD_FUNCTION(ptr, "_plugin_init")))
      {
	int e;
      	args = emalloc(sizeof(struct arglist));
      	arg_add_value(args, "preferences", ARG_ARGLIST, -1, (void*)preferences);
      	e = (*f_init)(args);
	if((e >= 0) && (arg_get_value(args, "NAME") != NULL))
	{
         plugin = emalloc(sizeof(struct arglist));   
 	 arg_add_value(plugin, "plugin_args", ARG_ARGLIST, -1, (void *)args);
         arg_add_value(plugin, "full_name", ARG_STRING, -1, estrdup(fullname));
 	 plug_set_launch(args, 0);
         prev_plugin = arg_get_value(plugins, name);
         if( prev_plugin == NULL )
          arg_add_value(plugins, name, ARG_ARGLIST, -1, (void *)plugin);
         else
          {
           plugin_free(prev_plugin);
           arg_set_value(plugins, name, -1, (void *)plugin);
          }
	}
	else
	{
	 arg_set_value(args, "preferences", -1, NULL);
	 arg_free_all(args);
	}
      }
    else log_write("Couldn't find the entry point in %s [%s]\n", name,LIB_LAST_ERROR());
    CLOSE_LIBRARY(ptr);
  }
 efree(&fullname);
 return(plugin);
}


int
nes_plugin_launch(globals, plugin, hostinfos, preferences, kb, name, soc)
	struct arglist * globals;
	struct arglist * plugin;
	struct arglist * hostinfos;
	struct arglist * preferences;
	struct arglist * kb; /* knowledge base */
	char * name;
	int soc;
{
 struct arglist * args;
 short* ports = arg_get_value(globals, "ports");
 int ports_num =(int)arg_get_value(globals, "ports_num");
 nthread_t module;
 plugin_run_t func = NULL;
 ext_library_t ptr = NULL;
 
 
 ptr = LOAD_LIBRARY(name);
 if(!ptr)
 	return -1;
 func = (plugin_run_t)LOAD_FUNCTION(ptr, "plugin_run");
 if(!func)func = (plugin_run_t)LOAD_FUNCTION(ptr, "_plugin_run");
 if(!func){
 	log_write("no 'plugin_run()' function in %s\n", name);
	return -1;
	}

 args = arg_get_value(plugin, "plugin_args");
 arg_add_value(args, "globals", ARG_ARGLIST, -1, globals);
 arg_add_value(args, "HOSTNAME", ARG_ARGLIST, -1, hostinfos);
 arg_add_value(args, "ports", ARG_PTR, -1, ports);
 arg_add_value(args, "func", ARG_PTR, -1, func);
 arg_add_value(args, "name", ARG_STRING, strlen(name), name);
 arg_add_value(args, "ports_num", ARG_INT, sizeof(int), (void*)ports_num);
 arg_set_value(args, "preferences", -1, preferences);
 arg_add_value(args, "pipe", ARG_INT, sizeof(int), (void*)soc);
 arg_add_value(args, "key", ARG_ARGLIST, -1, kb);
 module = create_thread((void*)nes_thread, args, (struct arglist*)-1);
 CLOSE_LIBRARY(ptr);
 return module;
}

static int nes_thread(args)
 struct arglist * args;
{
 int soc = (int)arg_get_value(args, "pipe");
 int soc2 = (int)arg_get_value(args, "SOCKET");
 struct arglist * globals = arg_get_value(args, "globals");
 int i;
 plugin_run_t func;
 
 /* XXX ugly hack */
 arg_set_value(globals, "global_socket", sizeof(int), (void*)soc2);
 for(i=3;i<256;i++)
 {
  if( ( i != soc )  && (i != soc2 ) )
    close(i);
 }
#ifdef HAVE_SETPROCTITLE
 setproctitle("testing %s (%s)", (char*)arg_get_value(arg_get_value(args, "HOSTNAME"), "NAME"), (char*)arg_get_value(args, "name"));
#endif 
 func = arg_get_value(args, "func");
 return func(args);
}

pl_class_t nes_plugin_class = {
    NULL,
    ".nes",
    nes_plugin_init,
    nes_plugin_add,
    nes_plugin_launch,
};
